﻿using System;
using System.ComponentModel;
using System.Linq.Expressions;
using System.Windows;
using System.Windows.Controls;
using Infragistics.Controls.Schedules;

namespace SchedulingTool
{
    public class CustomSchedulerDialogFactory : ScheduleDialogFactoryBase, INotifyPropertyChanged
    {
        #region Fields
        private object _dialogContent;
        #endregion

        #region Properties
        public object DialogContent
        {
            get { return this._dialogContent; }
            set { this.SetProperty(() => this.DialogContent, ref this._dialogContent, value); }
        }
        #endregion

        #region Overrides
        public override ActivityTypes SupportedActivityDialogTypes
        {
            get { return ActivityTypes.All; }
        }

        public override FrameworkElement CreateActivityDialog(FrameworkElement container,
                                                              XamScheduleDataManager dataManager, ActivityBase activity,
                                                              bool allowModifications, bool allowRemove)
        {
            return new ContentControl {Content = this.DialogContent};
        }
        #endregion

        #region INotifyPropertyChanged Members
        public event PropertyChangedEventHandler PropertyChanged;
        #endregion

        protected void NotifyPropertyChanged<TProperty>(Expression<Func<TProperty>> property)
        {
            if (this.PropertyChanged == null)
                return;

            var lambda = (LambdaExpression) property;
            var unaryExpression = lambda.Body as UnaryExpression;
            MemberExpression memberExpression = unaryExpression != null
                                                    ? (MemberExpression) unaryExpression.Operand
                                                    : (MemberExpression) lambda.Body;
            this.PropertyChanged(this, new PropertyChangedEventArgs(memberExpression.Member.Name));
        }

        protected void SetProperty<TProperty>(Expression<Func<TProperty>> property, ref TProperty field, TProperty value)
        {
            field = value;
            this.NotifyPropertyChanged(property);
        }
    }
}