﻿using System;
using System.Collections;
using System.Collections.ObjectModel;
using System.Collections.Specialized;
using System.ComponentModel;
using System.Globalization;
using System.Linq;
using System.Linq.Expressions;
using System.Reflection;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Input;
using System.Windows.Markup;
using Infragistics.Controls.Schedules;

namespace SchedulingTool
{
    /// <summary>
    /// Interaction logic for MyWrappedScheduler.xaml
    /// </summary>
    public partial class MyWrappedScheduler : UserControl, INotifyPropertyChanged
    {
        #region Fields
        private PropertyInfo _calendarIdProperty;
        private CustomSchedulerDialogFactory _dialogFactory;
        private PropertyInfo _resourceIdProperty;
        #endregion

        #region Properties

        #region DateNavigator
        public static readonly DependencyProperty DateNavigatorProperty =
            DependencyProperty.Register("DateNavigator", typeof (IOutlookDateNavigator), typeof (MyWrappedScheduler),
                                        new UIPropertyMetadata());

        public IOutlookDateNavigator DateNavigator
        {
            get { return (IOutlookDateNavigator) this.GetValue(DateNavigatorProperty); }
            set { this.SetValue(DateNavigatorProperty, value); }
        }
        #endregion

        #region ResourceMembers
        public static readonly DependencyProperty ResourceMembersProperty =
            DependencyProperty.Register("ResourceMembers", typeof (IEnumerable), typeof (MyWrappedScheduler),
                                        new UIPropertyMetadata());
       
        public IEnumerable ResourceMembers
        {
            get { return (IEnumerable) this.GetValue(ResourceMembersProperty); }
            set { this.SetValue(ResourceMembersProperty, value); }
        }
        #endregion

        #region ResourceCalendars
        public static readonly DependencyProperty ResourceCalendarsProperty =
            DependencyProperty.Register("ResourceCalendars", typeof (IEnumerable), typeof (MyWrappedScheduler),
                                        new UIPropertyMetadata(OnResourceCalendarPropertyChanged));

        public IEnumerable ResourceCalendars
        {
            get { return (IEnumerable) this.GetValue(ResourceCalendarsProperty); }
            set { this.SetValue(ResourceCalendarsProperty, value); }
        }
        #endregion

        #region Appointments
        public static readonly DependencyProperty AppointmentsProperty =
            DependencyProperty.Register("Appointments", typeof (IEnumerable), typeof (MyWrappedScheduler),
                                        new UIPropertyMetadata());

        public IEnumerable Appointments
        {
            get { return (IEnumerable) this.GetValue(AppointmentsProperty); }
            set { this.SetValue(AppointmentsProperty, value); }
        }
        #endregion

        #region DialogContent
        public static readonly DependencyProperty DialogContentProperty =
            DependencyProperty.Register("DialogContent", typeof (object), typeof (MyWrappedScheduler),
                                        new UIPropertyMetadata(OnDialogContentChanged));

        public object DialogContent
        {
            get { return this.GetValue(DialogContentProperty); }
            set { this.SetValue(DialogContentProperty, value); }
        }
        #endregion

        #region DialogFactory
        public CustomSchedulerDialogFactory DialogFactory
        {
            get { return this._dialogFactory; }
            set { this.SetProperty(() => this.DialogFactory, ref this._dialogFactory, value); }
        }
        #endregion

        #region AvailableTimeSlots
        public ObservableCollection<CalendarDateRange> AvailableTimeSlots
        {
            get { return (ObservableCollection<CalendarDateRange>)GetValue(AvailableTimeSlotsProperty); }
            set { SetValue(AvailableTimeSlotsProperty, value); }
        }

        // Using a DependencyProperty as the backing store for AvailableTimeSlots.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty AvailableTimeSlotsProperty =
            DependencyProperty.Register("AvailableTimeSlots", typeof(ObservableCollection<CalendarDateRange>), typeof(MyWrappedScheduler), new UIPropertyMetadata());        
        #endregion

        #endregion

        #region Commands
        public static readonly DependencyProperty DisplayDialogCommandProperty =
            DependencyProperty.Register("DisplayDialogCommand", typeof (ICommand), typeof (MyWrappedScheduler));

        public ICommand DisplayDialogCommand
        {
            get { return (ICommand) this.GetValue(DisplayDialogCommandProperty); }
            set { this.SetValue(DisplayDialogCommandProperty, value); }
        }
        #endregion

        #region Ctor
        public MyWrappedScheduler()
        {
            this.InitializeComponent();         
            //temp
            this.AvailableTimeSlots = new ObservableCollection<CalendarDateRange>
                                         {
                                              new CalendarDateRange(DateTime.Now,DateTime.Now.AddHours(.5)),
                                              new CalendarDateRange(DateTime.Now.AddHours(2),DateTime.Now.AddHours(2.5)),
                                              new CalendarDateRange(DateTime.Now.AddHours(4),DateTime.Now.AddHours(5)),
                                               new CalendarDateRange(DateTime.Now.AddHours(7),DateTime.Now.AddHours(7.45))                                              
                                         }; 
        }
        #endregion

        #region EventHandlers
        private static void OnResourceCalendarPropertyChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            var control = (MyWrappedScheduler) d;
            if (e.NewValue is INotifyCollectionChanged)
                control.OnCollectionChanged((INotifyCollectionChanged) e.NewValue);
        }

        public void OnCollectionChanged(INotifyCollectionChanged collection)
        {
            this._resourceIdProperty = null;
            this._calendarIdProperty = null;
            if (collection == null) return;
            collection.CollectionChanged += this.OnCalendarsChanged;
        }

        private void OnCalendarsChanged(object sender, NotifyCollectionChangedEventArgs e)
        {
            if (e.Action != NotifyCollectionChangedAction.Add || e.NewItems == null) return;
            if (this._resourceIdProperty == null && e.NewItems != null && e.NewItems.Count > 0 && e.NewItems[0] != null)
                this._resourceIdProperty = e.NewItems[0].GetType().GetProperty("OwningResourceId");
            if (this._calendarIdProperty == null && e.NewItems != null && e.NewItems.Count > 0 && e.NewItems[0] != null)
                this._calendarIdProperty = e.NewItems[0].GetType().GetProperty("Id");
            if (this._calendarIdProperty == null || this._resourceIdProperty == null) return;
            foreach (object item in e.NewItems.Cast<object>().Where(item => item != null))
            {
                this.xamScheduleDataManager1.CalendarGroups.Add(new CalendarGroup
                                                                    {
                                                                        InitialCalendarIds =
                                                                            string.Format("[{0}[{1}]]",
                                                                                          this._resourceIdProperty.
                                                                                              GetValue(item, null),
                                                                                          this._calendarIdProperty.
                                                                                              GetValue(item, null))
                                                                    });
            }
            this.NotifyPropertyChanged(() => this.ResourceCalendars);
        }

        private void OnActivityDialogDisplaying(object sender, ActivityDialogDisplayingEventArgs e)
        {
            if (this.DisplayDialogCommand != null)
                this.DisplayDialogCommand.Execute(e.Activity.DataItem);
        }

        private static void OnDialogContentChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            var control = (MyWrappedScheduler) d;
            if (e.NewValue != null && control != null)
            {
                control.DialogFactory = new CustomSchedulerDialogFactory {DialogContent = e.NewValue};
            }
        }
        #endregion

        #region INotifyPropertyChanged Members
        public event PropertyChangedEventHandler PropertyChanged;
        #endregion

        protected void NotifyPropertyChanged<TProperty>(Expression<Func<TProperty>> property)
        {
            if (this.PropertyChanged == null)
                return;

            var lambda = (LambdaExpression) property;
            var unaryExpression = lambda.Body as UnaryExpression;
            MemberExpression memberExpression = unaryExpression != null
                                                    ? (MemberExpression) unaryExpression.Operand
                                                    : (MemberExpression) lambda.Body;
            this.PropertyChanged(this, new PropertyChangedEventArgs(memberExpression.Member.Name));
        }

        protected void SetProperty<TProperty>(Expression<Func<TProperty>> property, ref TProperty field, TProperty value)
        {
            field = value;
            this.NotifyPropertyChanged(property);
        }
    }

    [MarkupExtensionReturnType(typeof(IValueConverter))]
    public class AvailabeTimeSlotConverter : MarkupExtension,IMultiValueConverter
    {
        public override object ProvideValue(IServiceProvider serviceProvider)
        {
            return this;
        }

        public object Convert(object[] values, Type targetType, object parameter, CultureInfo culture)
        {
            if (values == null || values.Count() != 2 || !(values[0] is DateTime) || !(values[1] is IEnumerable)) return false;
            var range =  (ObservableCollection<CalendarDateRange>) values[1];
            var start = (DateTime) values[0];
            return range.Any(date => start.Date == date.Start.Date && start.TimeOfDay >= date.Start.TimeOfDay && start.TimeOfDay <= date.End.TimeOfDay);
        }

        public object[] ConvertBack(object value, Type[] targetTypes, object parameter, CultureInfo culture)
        {
            throw new NotImplementedException();
        }
    }
}