﻿using System;
using System.ComponentModel;
using System.Linq;
using System.Linq.Expressions;
using WpfApplication1;

namespace SchedulingTool.ViewModels
{
    public class MySchedulingViewModel : INotifyPropertyChanged
    {
        #region Fields
        private readonly MyResourceList _resourceMembers;
        private RelayCommand<string> _addResourceCommand;
        private RelayCommand _loadResourcesCommand;
        #endregion

        #region Properties
        public MyResourceList ResourceMembers
        {
            get { return this._resourceMembers; }
        }
        #endregion

        #region Commands
        public RelayCommand LoadResourcesCommand
        {
            get { return this._loadResourcesCommand; }
            internal set
            {
                this._loadResourcesCommand = value;
                this.NotifyPropertyChanged(() => this.LoadResourcesCommand);
            }
        }

        public RelayCommand<string> AddResourceCommand
        {
            get { return this._addResourceCommand; }
            internal set
            {
                this._addResourceCommand = value;
                this.NotifyPropertyChanged(() => this.AddResourceCommand);
            }
        }
        #endregion

        #region Ctor
        public MySchedulingViewModel()
        {
            this._resourceMembers = new MyResourceList();
            this.AddResourceCommand = new RelayCommand<string>(this.AddResource, this.CanAddResource);
            this.LoadResourcesCommand = new RelayCommand(this.LoadResources, this.CanLoadResources);
        }
        #endregion

        #region INotifyPropertyChanged Members
        public event PropertyChangedEventHandler PropertyChanged;
        #endregion

        protected void NotifyPropertyChanged<TProperty>(Expression<Func<TProperty>> property)
        {
            if (this.PropertyChanged == null)
                return;

            var lambda = (LambdaExpression) property;
            var unaryExpression = lambda.Body as UnaryExpression;
            MemberExpression memberExpression = unaryExpression != null
                                                    ? (MemberExpression) unaryExpression.Operand
                                                    : (MemberExpression) lambda.Body;
            this.PropertyChanged(this, new PropertyChangedEventArgs(memberExpression.Member.Name));
        }

        protected void SetProperty<TProperty>(Expression<Func<TProperty>> property, ref TProperty field, TProperty value)
        {
            field = value;
            this.NotifyPropertyChanged(property);
        }

        #region Methods
        private bool CanLoadResources()
        {
            return true;
        }

        private void LoadResources()
        {
            this.ResourceMembers.Add(new MyResource {Id = "David", IsVisible = true, Name = "David"});
            this.ResourceMembers.Add(new MyResource {Id = "Kim", IsVisible = true, Name = "Kim"});
            this.ResourceMembers.Add(new MyResource {Id = "Harry", IsVisible = true, Name = "Harry"});
        }

        private bool CanAddResource(string name)
        {
            return this.ResourceMembers.FirstOrDefault(item => String.Compare(item.Id, name) == 0) == null;
        }

        private void AddResource(string name)
        {
            this.ResourceMembers.Add(new MyResource {Id = name, IsVisible = true, Name = name});
        }
        #endregion
    }
}